@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2003 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
  *                 All Rights Reserved
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -21,23 +21,87 @@
 #ifndef COMMON_H
 #define COMMON_H
 
+/**************************************** Common constants */
+
+#ifndef VERSION
+#define VERSION "4.18"
+#endif
+
+#ifdef OPTIMIZE_SCALABILITY
+
+/* CPU stack size */
+#define STACK_SIZE      32768
+#define DEBUG_STACK_SIZE
+
+/* I/O buffer size */
+#define BUFFSIZE        2048
+
+/* Length of strings (including the terminating '\0' character) */
+#define STRLEN          128
+
+#else /* OPTIMIZE_SCALABILITY */
+
+/* CPU stack size */
+#define STACK_SIZE      65536
+
+/* I/O buffer size */
+#define BUFFSIZE        16384
+
+/* Length of strings (including the terminating '\0' character) */
+#define STRLEN          256
+
+#endif /* OPTIMIZE_SCALABILITY */
+
+/* IP address and TCP port textual representation length */
+#define IPLEN           128
+
+/* How many bytes of random input to read from files for PRNG */
+/* OpenSSL likes at least 128 bits, so 64 bytes seems plenty. */
+#define RANDOM_BYTES    64
+
 /* For FormatGuard */
 /* #define __NO_FORMATGUARD_ */
 
-#ifndef VERSION
-#define VERSION "4.04"
+/**************************************** Platform */
+
+#ifdef USE_WIN32
+#define USE_IPv6
 #endif
 
-#ifndef USE_WIN32
+#ifdef _WIN32_WCE
+#define USE_WIN32
+typedef int socklen_t;
+#define EINTR WSAEINTR
+#define EMFILE WSAEMFILE
+#endif
 
-/* POSIX threads */
-#if HAVE_PTHREAD_H && HAVE_LIBPTHREAD
-#define USE_PTHREAD
+#ifdef USE_WIN32
+#define HAVE_OPENSSL
+#define HAVE_OSSL_ENGINE_H
+#endif
+
+/**************************************** Generic headers */
+
+#ifdef __vms
+#include <starlet.h>
+#endif /* __vms */
+
+/* For nsr-tandem-nsk architecture */
+#ifdef __TANDEM
+#include <floss.h>
+#endif
+
+/* threads model */
+#ifdef USE_UCONTEXT
+#define __MAKECONTEXT_V2_SOURCE
+#include <ucontext.h>
+#endif
+
+#ifdef USE_PTHREAD
 #define THREADS
 #define _REENTRANT
 #define _THREAD_SAFE
-#else
-#define USE_FORK
+#include <pthread.h>
 #endif
 
 /* TCP wrapper */
@@ -45,13 +109,11 @@
 #define USE_LIBWRAP
 #endif
 
-#endif /* !USE_WIN32 */
-
 /* Must be included before sys/stat.h for Ultrix */
 #include <sys/types.h>   /* u_short, u_long */
-
 /* General headers */
 #include <stdio.h>
+/* Must be included before sys/stat.h for Ultrix */
 #include <errno.h>
 #include <stdlib.h>
 #include <stdarg.h>      /* va_ */
@@ -59,36 +121,31 @@
 #include <ctype.h>       /* isalnum */
 #include <time.h>
 #include <sys/stat.h>    /* stat */
+#include <setjmp.h>
 
-#if defined (USE_WIN32) || defined (__vms)
-#define LOG_EMERG       0
-#define LOG_ALERT       1
-#define LOG_CRIT        2
-#define LOG_ERR         3
-#define LOG_WARNING     4
-#define LOG_NOTICE      5
-#define LOG_INFO        6
-#define LOG_DEBUG       7
-#endif /* defined (USE_WIN32) || defined (__vms) */
+/**************************************** WIN32 headers */
 
 #ifdef USE_WIN32
 
+#ifndef HOST
 #ifdef __MINGW32__
 #define HOST "x86-pc-mingw32-gnu"
 #else
 #define HOST "x86-pc-unknown"
 #endif
+#endif
 
 typedef unsigned char u8;
 typedef unsigned short u16;
 typedef unsigned long u32;
 
+#define HAVE_SNPRINTF
+#define snprintf _snprintf
 #define HAVE_VSNPRINTF
 #define vsnprintf _vsnprintf
-/*  Already defined for mingw, perhaps others
-int _vsnprintf(char *, int, char *, ...);
-*/
-#define strcasecmp stricmp
+#define strcasecmp _stricmp
+#define strncasecmp _strnicmp
+#define sleep(c) Sleep(1000*(c))
 
 #define exit(c) exit_stunnel(c)
 
@@ -97,10 +154,17 @@ int _vsnprintf(char *, int, char *, ...);
 #define readsocket(s,b,n)       recv((s),(b),(n),0)
 #define writesocket(s,b,n)      send((s),(b),(n),0)
 
-#define __USE_W32_SOCKETS
-
 /* #define FD_SETSIZE 4096 */
 /* #define Win32_Winsock */
+#define __USE_W32_SOCKETS
+
+/* Winsock2 header for IPv6 definitions */
+#ifdef _WIN32_WCE
+#include <winsock.h>
+#else
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#endif
 #include <windows.h>
 
 #define ECONNRESET WSAECONNRESET
@@ -112,9 +176,14 @@ int _vsnprintf(char *, int, char *, ...);
 #undef EINVAL
 #define EINVAL WSAEINVAL
 
+#include <process.h>     /* _beginthread */
+#include <tchar.h>
+
 #define NO_IDEA
 #define OPENSSL_NO_IDEA
 
+/**************************************** non-WIN32 headers */
+
 #else /* USE_WIN32 */
 
 #if SIZEOF_UNSIGNED_CHAR == 1
@@ -133,13 +202,21 @@ typedef unsigned int u32;
 typedef unsigned long u32;
 #endif
 
+#ifdef __INNOTEK_LIBC__
+# define get_last_socket_error() sock_errno()
+# define get_last_error()        errno
+# define readsocket(s,b,n)       recv((s),(b),(n),0)
+# define writesocket(s,b,n)      send((s),(b),(n),0)
+# define closesocket(s)          close(s)
+# define ioctlsocket(a,b,c)      so_ioctl((a),(b),(c))
+#else
 #define get_last_socket_error() errno
 #define get_last_error()        errno
 #define readsocket(s,b,n)       read((s),(b),(n))
 #define writesocket(s,b,n)      write((s),(b),(n))
 #define closesocket(s)          close(s)
 #define ioctlsocket(a,b,c)      ioctl((a),(b),(c))
-
+#endif
     /* OpenVMS compatibility */
 #ifdef __vms
 #define libdir "__NA__"
@@ -170,11 +247,24 @@ typedef unsigned long u32;
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>  /* for aix */
 #endif
+
+#ifndef BROKEN_POLL
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#define USE_POLL
+#else /* HAVE_POLL_H */
+#ifdef HAVE_SYS_POLL_H
+#include <sys/poll.h>
+#define USE_POLL
+#endif /* HAVE_SYS_POLL_H */
+#endif /* HAVE_POLL_H */
+#endif /* BROKEN_POLL */
+
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>   /* for FIONBIO */
 #endif
 #include <pwd.h>
-#ifdef HAVE_SETGROUPS
+#ifdef HAVE_GRP_H
 #include <grp.h>
 #endif
 #ifdef __BEOS__
@@ -182,7 +272,6 @@ typedef unsigned long u32;
 #endif
 #include <fcntl.h>
 
-    /* Networking headers */
 #include <netinet/in.h>  /* struct sockaddr_in */
 #include <sys/socket.h>  /* getpeername */
 #include <arpa/inet.h>   /* inet_ntoa */
@@ -218,29 +307,28 @@ extern char *sys_errlist[];
 
 #endif /* USE_WIN32 */
 
-/* OpenSSL headers */
+/**************************************** OpenSSL headers */
+
 #ifdef HAVE_OPENSSL
 #include <openssl/lhash.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/crypto.h> /* for CRYPTO_* and SSLeay_version */
 #include <openssl/rand.h>
-#else
+#ifdef HAVE_OSSL_ENGINE_H
+#include <openssl/engine.h>
+#endif
+#if SSLEAY_VERSION_NUMBER >= 0x00907000L
+#include <openssl/ocsp.h>
+#endif /* OpenSSL-0.9.7 */
+#else /* HAVE_OPENSSL */
 #include <lhash.h>
 #include <ssl.h>
 #include <err.h>
 #include <crypto.h> /* for CRYPTO_* and SSLeay_version */
-#endif
+#endif /* HAVE_OPENSSL */
 
-/* I/O buffer size */
-#define BUFFSIZE 16384
-
-/* Length of strings (including the terminating '\0' character) */
-#define STRLEN       1024
-
-/* How many bytes of random input to read from files for PRNG */
-/* OpenSSL likes at least 128 bits, so 64 bytes seems plenty. */
-#define RANDOM_BYTES 64
+/**************************************** Other defines */
 
 /* Safe copy for strings declarated as char[STRLEN] */
 #define safecopy(dst, src) \
@@ -249,10 +337,39 @@ extern char *sys_errlist[];
     (dst[STRLEN-1]='\0', strncat((dst), (src), STRLEN-strlen(dst)-1))
 /* change all non-printable characters to '.' */
 #define safestring(s) \
-    do {char *p; for(p=(s); *p; p++) if(!isprint((int)*p)) *p='.';} while(0)
+    do {unsigned char *p; for(p=(s); *p; p++) \
+        if(!isprint((int)*p)) *p='.';} while(0)
 /* change all unsafe characters to '.' */
 #define safename(s) \
-    do {char *p; for(p=(s); *p; p++) if(!isalnum((int)*p)) *p='.';} while(0)
+    do {unsigned char *p; for(p=(s); *p; p++) \
+        if(!isalnum((int)*p)) *p='.';} while(0)
+
+/* Some definitions for IPv6 support */
+#if defined(USE_IPv6)
+#define addr_len(x) ((x).sa.sa_family==AF_INET ? \
+    sizeof(struct sockaddr_in) : sizeof(struct sockaddr_in6))
+#else
+#define addr_len(x) (sizeof(struct sockaddr_in))
+#endif
+
+/* Always use IPv4 defaults! */
+#define DEFAULT_LOOPBACK "127.0.0.1"
+#define DEFAULT_ANY "0.0.0.0"
+#if 0
+#define DEFAULT_LOOPBACK "::1"
+#define DEFAULT_ANY "::"
+#endif
+
+#if defined (USE_WIN32) || defined (__vms)
+#define LOG_EMERG       0
+#define LOG_ALERT       1
+#define LOG_CRIT        2
+#define LOG_ERR         3
+#define LOG_WARNING     4
+#define LOG_NOTICE      5
+#define LOG_INFO        6
+#define LOG_DEBUG       7
+#endif /* defined (USE_WIN32) || defined (__vms) */
 
 #endif /* defined COMMON_H */
 
